==> foundation-ui-components-1.25.0.tgz: package/src/common/select/NsSelect.tsx <==
import React, { MouseEvent, useCallback, useMemo, useRef, useState } from 'react';

import { useOnClickOutside } from '../hooks/useOnClickOutside';
import { NsIcon } from '../icon';
import { Show } from '../utility';
import {
    ErrorContainer,
    StyledArrowContainer,
    SelectButton,
    SelectContainer,
    SelectHead,
    SelectLabel,
    SelectList,
    SelectMain,
} from './NsSelect.styled';
import { LabelValueType, SelectProps } from './types.d';
import { arrowVisibility, factorMap } from './constant';
import { DefaultRenderOption } from './DefaultRenderOption';

const NsSelect: React.FC<SelectProps> = ({
    selected,
    onSelect,
    disabled = false,
    type = 'outlined',
    placeholder = '',
    selectBorderColor = 'accentColorB',
    closeOnMouseLeave = false,
    showListDivider = true,
    label,
    readOnly = false,
    errorMessage,
    hasArrows,
    // eslint-disable-next-line no-unused-vars
    rules,
    reverseArrowControls,
    RenderOption = DefaultRenderOption,
    options,
}) => {
    const [isOpen, setIsOpen] = useState(false);
    const optionIndex = useMemo(() => {
        if (typeof selected?.index === 'number') return selected.index;
        return options.findIndex((option) => option.value === selected.value);
    }, [options, selected]);
    const [value, setValue] = useState<LabelValueType>({ ...selected, index: optionIndex });
    const selectContainerRef = useRef<HTMLDivElement>(null);
    const selectHeadRef = useRef<HTMLDivElement>(null);

    const closeMenuOnMouseLeave = useCallback(() => {
        if (closeOnMouseLeave) {
            setIsOpen(false);
        }
    }, [closeOnMouseLeave]);

    const handleSelect = (val: LabelValueType) => {
        onSelect(val);
        setIsOpen(false);
        setValue(val);
    };
    const closeMenu = useCallback(() => {
        setIsOpen(false);
    }, []);
    useOnClickOutside(selectContainerRef.current, closeMenu);

    const handleMouseClick = () => {
        if (disabled) return;
        setIsOpen(!isOpen);
        if (!isOpen) {
            setTimeout(() => {
                selectHeadRef.current?.focus();
            }, 0);
        }
    };
    const handleArrowClick = (event: MouseEvent) => {
        const direction = event.currentTarget?.className.includes('Right') ? 'right' : 'left';
        const adder = factorMap[reverseArrowControls ? 'reverse' : 'forward'][direction];
        const nextIndex = (selected?.index ?? -1) + adder;
        if (nextIndex > -1 && nextIndex < options.length) {
            handleSelect({ ...options[nextIndex], index: nextIndex });
        }
    };
    return (
        <SelectMain>
            {hasArrows && (
                <StyledArrowContainer
                    onClick={handleArrowClick}
                    className="LeftArrow"
                    style={{
                        visibility: arrowVisibility(options.length, 'left', reverseArrowControls, value?.index),
                    }}
                >
                    <NsIcon name="arrowLeft" color="primary" />
                </StyledArrowContainer>
            )}
            <SelectContainer
                data-testid="select-container"
                disabled={disabled || readOnly}
                ref={selectContainerRef}
                errorMessage={errorMessage}
            >
                {label && <SelectLabel errorMessage={errorMessage}>{label}</SelectLabel>}
                <SelectHead
                    data-testid="select-head"
                    disabled={disabled || readOnly}
                    type={type}
                    onMouseDown={handleMouseClick}
                    tabIndex={0}
                    ref={selectHeadRef}
                >
                    <SelectButton>{value?.label || placeholder}</SelectButton>
                    {!readOnly && <NsIcon name={isOpen ? 'arrowUp' : 'arrowDown'} color="primary" />}
                </SelectHead>
                <Show when={isOpen}>
                    <SelectList
                        data-testid="select-menu-list"
                        onMouseLeave={closeMenuOnMouseLeave}
                        borderColor={selectBorderColor}
                        className="selectList"
                    >
                        {options.map((opt, optionIndex) => {
                            return (
                                <RenderOption
                                    key={opt.value}
                                    value={opt.value}
                                    label={opt.label}
                                    index={optionIndex}
                                    onClick={() =>
                                        handleSelect({ label: opt.label, value: opt.value, index: optionIndex })
                                    }
                                    selected={value}
                                    showListDivider={showListDivider}
                                />
                            );
                        })}
                    </SelectList>
                </Show>
                {errorMessage && <ErrorContainer>{errorMessage}</ErrorContainer>}
            </SelectContainer>
            {hasArrows && (
                <StyledArrowContainer
                    onClick={handleArrowClick}
                    className="RightArrow"
                    style={{
                        visibility: arrowVisibility(options.length, 'right', reverseArrowControls, selected?.index),
                    }}
                >
                    <NsIcon name="arrowRight" color="primary" />
                </StyledArrowContainer>
            )}
        </SelectMain>
    );
};

export default NsSelect;

==> foundation-ui-components-1.30.1.tgz: package/src/common/select/NsSelect.tsx <==
import React, {
    MouseEvent,
    ReactElement,
    useCallback,
    useMemo,
    useRef,
    useState,
} from 'react';

import { useOnClickOutside } from '../hooks/useOnClickOutside';
import { NsIcon } from '../icon';
import { Show } from '../utility';
import {
    arrowVisibility,
    factorMap,
} from './constant';
import { DefaultRenderOption } from './DefaultRenderOption';
import {
    ErrorContainer,
    SelectButton,
    SelectContainer,
    SelectHead,
    SelectLabel,
    SelectList,
    SelectMain,
    StyledArrowContainer,
} from './NsSelect.styled';
import {
    LabelValueType,
    SelectProps,
} from './types.d';

const NsSelect: React.FC<SelectProps> = ({
    selected,
    onSelect,
    disabled = false,
    type = 'outlined',
    placeholder = '',
    selectBorderColor = 'accentColorB',
    closeOnMouseLeave = false,
    showListDivider = true,
    label,
    readOnly = false,
    errorMessage,
    hasArrows,
    // eslint-disable-next-line no-unused-vars
    rules,
    reverseArrowControls,
    RenderOption = DefaultRenderOption,
    options,
}): ReactElement => {
    const [isOpen, setIsOpen] = useState(false);
    const optionIndex = useMemo(() => {
        if (typeof selected?.index === 'number') return selected.index;
        return options.findIndex((option) => option.value === selected.value);
    }, [options, selected]);
    const [value, setValue] = useState<LabelValueType>({ ...selected, index: optionIndex });
    const selectContainerRef = useRef<HTMLDivElement>(null);
    const selectHeadRef = useRef<HTMLDivElement>(null);

    const closeMenuOnMouseLeave = useCallback(() => {
        if (closeOnMouseLeave) {
            setIsOpen(false);
        }
    }, [closeOnMouseLeave]);

    const handleSelect = (val: LabelValueType) => {
        onSelect(val);
        setIsOpen(false);
        setValue(val);
    };
    const closeMenu = useCallback(() => {
        setIsOpen(false);
    }, []);
    useOnClickOutside(selectContainerRef.current, closeMenu);

    const handleMouseClick = () => {
        if (disabled) return;
        setIsOpen(!isOpen);
        if (!isOpen) {
            setTimeout(() => {
                selectHeadRef.current?.focus();
            }, 0);
        }
    };
    const handleArrowClick = (event: MouseEvent) => {
        const direction = event.currentTarget?.className.includes('Right') ? 'right' : 'left';
        const adder = factorMap[reverseArrowControls ? 'reverse' : 'forward'][direction];
        const nextIndex = (selected?.index ?? -1) + adder;
        if (nextIndex > -1 && nextIndex < options.length) {
            handleSelect({ ...options[nextIndex], index: nextIndex });
        }
    };
    return (
        <SelectMain>
            {hasArrows && (
                <StyledArrowContainer
                    onClick={handleArrowClick}
                    className="LeftArrow"
                    style={{
                        visibility: arrowVisibility(options.length, 'left', reverseArrowControls, value?.index),
                    }}
                >
                    <NsIcon name="arrowLeft" color="primary" />
                </StyledArrowContainer>
            )}
            <SelectContainer
                data-testid="select-container"
                disabled={disabled || readOnly}
                ref={selectContainerRef}
                errorMessage={errorMessage}
            >
                {label && <SelectLabel errorMessage={errorMessage}>{label}</SelectLabel>}
                <SelectHead
                    data-testid="select-head"
                    disabled={disabled || readOnly}
                    type={type}
                    onMouseDown={handleMouseClick}
                    tabIndex={0}
                    ref={selectHeadRef}
                >
                    <SelectButton>{value?.label || placeholder}</SelectButton>
                    {!readOnly && <NsIcon name={isOpen ? 'arrowUp' : 'arrowDown'} color="primary" />}
                </SelectHead>
                <Show when={isOpen}>
                    <SelectList
                        data-testid="select-menu-list"
                        onMouseLeave={closeMenuOnMouseLeave}
                        borderColor={selectBorderColor}
                        className="selectList"
                    >
                        {options.map((opt, optionIndex) => {
                            return (
                                <RenderOption
                                    key={opt.value}
                                    value={opt.value}
                                    label={opt.label}
                                    index={optionIndex}
                                    onClick={() =>
                                        handleSelect({ label: opt.label, value: opt.value, index: optionIndex })
                                    }
                                    selected={value}
                                    showListDivider={showListDivider}
                                />
                            );
                        })}
                    </SelectList>
                </Show>
                {errorMessage && <ErrorContainer>{errorMessage}</ErrorContainer>}
            </SelectContainer>
            {hasArrows && (
                <StyledArrowContainer
                    onClick={handleArrowClick}
                    className="RightArrow"
                    style={{
                        visibility: arrowVisibility(options.length, 'right', reverseArrowControls, selected?.index),
                    }}
                >
                    <NsIcon name="arrowRight" color="primary" />
                </StyledArrowContainer>
            )}
        </SelectMain>
    );
};

export default NsSelect;
